/*
 * Copyright 2017 aquenos GmbH.
 * All rights reserved.
 * 
 * Please refer to the file NOTICE.txt in the root of the source distribution or
 * in the META-INF directory of the binary archive for licensing information.
 */

package com.aquenos.epics.jackie.diirt.vtype;

import java.time.Instant;
import java.util.List;

import org.apache.commons.lang3.builder.EqualsBuilder;
import org.apache.commons.lang3.builder.HashCodeBuilder;
import org.apache.commons.lang3.builder.ToStringBuilder;

import com.aquenos.epics.jackie.common.value.ChannelAccessGraphicsEnum;
import com.aquenos.epics.jackie.common.value.ChannelAccessTimeEnum;

/**
 * Base class for {@link JackieVEnum} and {@link JackieVEnumArray}. It
 * implements the method related to meta-data because the logic is the same for
 * both types.
 * 
 * @author Sebastian Marsching
 */
public abstract class JackieEnum
        extends JackieAlarmAndTime<ChannelAccessTimeEnum>
        implements org.diirt.vtype.Enum {

    /**
     * Backing value that supplies the meta-data for this instance.
     */
    protected final ChannelAccessGraphicsEnum graphicsValue;

    /**
     * <p>
     * Initializes this value with the specified backing values. The
     * {@code graphicsValue} is used for the meta-data (labels) and the
     * {@code timeValue} is used for the alarm status and time-stamp. If the
     * {@code scalar} flag is set, the {@code timeValue} must have exactly one
     * element.
     * </p>
     * 
     * <p>
     * If {@code disconnectTime} is not <code>null</code>, it is used as the
     * time-stamp and the alarm severity is set to {@code UNDEFINED}.
     * </p>
     * 
     * <p>
     * This constructor is package-private because the classes implementing the
     * VTypes form a closed hierarchy.
     * </p>
     * 
     * @param graphicsValue
     *            value used for meta-data (labels).
     * @param timeValue
     *            value used for alarm status, time-stamp, and value.
     * @param disconnectTime
     *            time at which the channel was disconnected. Usually this
     *            should be <code>null</code>. If it is not <code>null</code>,
     *            the alarm severity is set to {@code UNDEFINED} and this
     *            parameter is used as the time-stamp (instead of using the
     *            time-stamp from the {@code timeValue}.
     * @param scalar
     *            <code>true</code> if the child class represents a scalar,
     *            <code>false</code> if it represents an array.
     * @throws IllegalArgumentException
     *             if {@code scalar} is <code>true</code> and {@code timeValue}
     *             has more or less than one element.
     */
    JackieEnum(ChannelAccessGraphicsEnum graphicsValue,
            ChannelAccessTimeEnum timeValue, Instant disconnectTime,
            boolean scalar) {
        super(timeValue, disconnectTime, scalar);
        if (graphicsValue == null) {
            throw new NullPointerException();
        }
        this.graphicsValue = graphicsValue;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null || !super.equals(obj)) {
            return false;
        }
        // The equals(...) implementation in the super-class already checks that
        // the types match, so we can cast without further tests.
        JackieEnum other = (JackieEnum) obj;
        return new EqualsBuilder()
                .append(this.graphicsValue, other.graphicsValue).build();
    }

    @Override
    public int hashCode() {
        return new HashCodeBuilder(97, 31).appendSuper(super.hashCode())
                .append(graphicsValue).build();
    }

    @Override
    public String toString() {
        return new ToStringBuilder(this).append("graphicsValue", graphicsValue)
                .append("disconnectTime").append("scalar", scalar)
                .append("timeValue", timeValue).build();
    }

    @Override
    public List<String> getLabels() {
        return graphicsValue.getLabels();
    }

}
