/*
 * Copyright 2017 aquenos GmbH.
 * All rights reserved.
 * 
 * Please refer to the file NOTICE.txt in the root of the source distribution or
 * in the META-INF directory of the binary archive for licensing information.
 */

package com.aquenos.epics.jackie.diirt.vtype;

import java.time.Instant;
import java.util.List;

import org.diirt.util.array.ArrayInt;
import org.diirt.util.array.ListInt;
import org.diirt.vtype.ArrayDimensionDisplay;
import org.diirt.vtype.VNumberArray;
import org.diirt.vtype.ValueUtil;

import com.aquenos.epics.jackie.common.value.ChannelAccessNumericControlsValue;
import com.aquenos.epics.jackie.common.value.ChannelAccessTimeValue;

/**
 * Base class for {@link JackieVByteArray}, {@link JackieVDoubleArray},
 * {@link JackieVFloatArray}, {@link JackieVIntArray}, and
 * {@link JackieVShortArray}. It implements the method related to array sizes
 * because the logic is the same for all these types.
 * 
 * @author Sebastian Marsching
 */
public abstract class JackieVNumberArray<ControlsValueType extends ChannelAccessNumericControlsValue<?>, TimeValueType extends ChannelAccessTimeValue<?>>
        extends JackieDisplay<ControlsValueType, TimeValueType>
        implements VNumberArray {

    /**
     * <p>
     * Initializes this value with the specified backing values. The
     * {@code controlsValue} is used for the meta-data (engineering units,
     * limits, etc.) and the {@code timeValue} is used for the alarm status,
     * time-stamp, and value.
     * </p>
     * 
     * <p>
     * If {@code disconnectTime} is not <code>null</code>, it is used as the
     * time-stamp and the alarm severity is set to {@code UNDEFINED}.
     * </p>
     * 
     * <p>
     * This constructor is package-private because the classes implementing the
     * VTypes form a closed hierarchy.
     * </p>
     * 
     * @param controlsValue
     *            value used for meta-data (engineering units, limits, etc.).
     * @param timeValue
     *            value used for alarm status, time-stamp, and value.
     * @param disconnectTime
     *            time at which the channel was disconnected. Usually this
     *            should be <code>null</code>. If it is not <code>null</code>,
     *            the alarm severity is set to {@code UNDEFINED} and this
     *            parameter is used as the time-stamp (instead of using the
     *            time-stamp from the {@code timeValue}.
     * @param honorZeroPrecision
     *            defines how the value is formatted if the
     *            {@code controlsValue}'s precision is zero. If
     *            <code>true</code> a precision of zero means that no decimal
     *            digits shall be printed. If <code>false</code>, a precision of
     *            zero means that a default number of decimal digits shall be
     *            printed. This parameter is only relevant for the return value
     *            of the {@link #getFormat()} method.
     */
    JackieVNumberArray(ControlsValueType controlsValue, TimeValueType timeValue,
            Instant disconnectTime, boolean honorZeroPrecision) {
        super(controlsValue, timeValue, disconnectTime, honorZeroPrecision,
                false);
    }

    @Override
    public ListInt getSizes() {
        return new ArrayInt(timeValue.getValueSize());
    }

    @Override
    public List<ArrayDimensionDisplay> getDimensionDisplay() {
        return ValueUtil.defaultArrayDisplay(getSizes());
    }

}
